@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2023. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -30,7 +30,9 @@
 	 otp_7422/1, zero_width/1, bad_append/1, bs_append_overflow/1,
          bs_append_offheap/1,
          reductions/1, fp16/1, zero_init/1, error_info/1, little/1,
-         heap_binary_unit/1]).
+         heap_binary_unit/1,
+         otp_24_code_gh_8238/1,
+         many_segments/1]).
 
 -include_lib("common_test/include/ct.hrl").
 
@@ -45,7 +47,9 @@ all() ->
      copy_writable_binary, kostis, dynamic, bs_add, otp_7422, zero_width,
      bad_append, bs_append_overflow, bs_append_offheap,
      reductions, fp16, zero_init,
-     error_info, little, heap_binary_unit].
+     error_info, little, heap_binary_unit,
+     otp_24_code_gh_8238,
+     many_segments].
 
 init_per_suite(Config) ->
     Config.
@@ -780,10 +784,16 @@ dynamic_3(Bef, N, {Int0,Lpad,Rpad,Dynamic}=Data, Count) ->
     Dynamic(Bef, N, Int1, Lpad, Rpad),
     Dynamic(Bef, N, -Int1, Lpad, Rpad),
 
-    %% OTP-7085: Test a small number in a wide field.
+    %% OTP-7085: Test a small number in a wide segment.
     Int2 = Int0 band 16#FFFFFF,
     Dynamic(Bef, N, Int2, Lpad, Rpad),
     Dynamic(Bef, N, -Int2, Lpad, Rpad),
+
+    %% Test a bignum in a short segment.
+    Int4 = ((Lpad bxor Rpad) bsl N) bor Int0,
+    Dynamic(Bef, N, Int4, Lpad, Rpad),
+    Dynamic(Bef, N, -Int4, Lpad, Rpad),
+
     dynamic_3(Bef, N-1, Data, Count+1).
 
 dynamic_big(Bef, N, Int, Lpad, Rpad) ->
@@ -1389,6 +1399,16 @@ do_zero_init_1(Size, LPad, RPad) ->
         end()).
 
 error_info(_Config) ->
+    case ?MODULE of
+        bs_construct_r24_SUITE ->
+            %% Error information is not implemented for old bit syntax
+            %% instructions.
+            ok;
+        _ ->
+            error_info()
+    end.
+
+error_info() ->
     Atom = id(some_atom),
     NegSize = id(-1),
     HugeNegSize = id(-1 bsl 64),
@@ -1696,6 +1716,285 @@ heap_binary_unit_2(Variant, Rest) ->
             {error2, Bin2}
     end.
 
+otp_24_code_gh_8238(Config) ->
+    case ?MODULE of
+        bs_construct_SUITE ->
+            %% GH-8238. Code compiled with Erlang/OTP 24 would crash
+            %% when run on OTP-26.2.3.
+            DataDir = proplists:get_value(data_dir, Config),
+            Asm = filename:join(DataDir, atom_to_list(?FUNCTION_NAME) ++ ".S"),
+            {ok,Mod,Beam} = compile:file(Asm, [binary,from_asm,report]),
+            {module,Mod} = code:load_binary(Mod, "", Beam),
+            Mod:Mod(),
+            ok;
+        _ ->
+            {skip,"Enough to run once"}
+    end.
+
+%% GH-8815: Binary construction with "too many" segments failed to JIT on ARM.
+many_segments(_Config) ->
+    Val = id(<<"fhqwhgads">>),
+    id(<<"COL_A,COL_B\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n", Val/binary, ",B0\n",
+         Val/binary, ",B0\n">>),
+    ok.
+
 %%%
 %%% Common utilities.
 %%%
