@@ -1,7 +1,7 @@
 %
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2005-2020. All Rights Reserved.
+%% Copyright Ericsson AB 2005-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -81,8 +81,9 @@
 -include_lib("common_test/include/ct.hrl").
 -include_lib("kernel/include/file.hrl").
 -include("ssh_test_lib.hrl").
-						% Default timetrap timeout
--define(default_timeout, ?t:minutes(1)).
+-include_lib("stdlib/include/assert.hrl").
+%% Default timetrap timeout
+-define(default_timeout, test_server:minutes(0.5)).
 
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
@@ -90,7 +91,7 @@
 
 suite() ->
     [{ct_hooks,[ts_install_cth]},
-     {timetrap,{seconds,40}}].
+     {timetrap,{seconds,20}}].
 
 all() -> 
     [{group, not_unicode},
@@ -119,7 +120,8 @@ groups() ->
 
      {unicode, [], [{group,erlang_server},
 		    {group,openssh_server},
-		    sftp_nonexistent_subsystem]},
+		    sftp_nonexistent_subsystem
+                   ]},
 
      {big_recvpkt_size, [], [{group,erlang_server},
 			     {group,openssh_server}]},
@@ -231,7 +233,9 @@ init_per_group(openssh_server, Config) ->
     Host = ssh_test_lib:hostname(),
     case (catch ssh_sftp:start_channel(Host,
 				       [{user_interaction, false},
-					{silently_accept_hosts, true}])) of
+					{silently_accept_hosts, true},
+                                        {save_accepted_host, false}
+                                       ])) of
 	{ok, _ChannelPid, Connection} ->
 	    [{peer, {_HostName,{IPx,Portx}}}] = ssh:connection_info(Connection,[peer]),
 	    ssh:close(Connection),
@@ -250,16 +254,16 @@ init_per_group(remote_tar, Config) ->
     ct:log("Server (~p) at ~p:~p",[proplists:get_value(group,Config),Host,Port]),
     User = proplists:get_value(user, Config),
     Passwd = proplists:get_value(passwd, Config),
-    {ok, Connection} =
+    Connection =
 	case proplists:get_value(group, Config) of
 	    erlang_server ->
-		ssh:connect(Host, Port,
+		ssh_test_lib:connect(Host, Port,
 			    [{user, User},
 			     {password, Passwd},
 			     {user_interaction, false},
 			     {silently_accept_hosts, true}]);
 	    openssh_server ->
-		ssh:connect(Host, Port,
+		ssh_test_lib:connect(Host, Port,
 			    [{user_interaction, false},
 			     {silently_accept_hosts, true}])
 	end,
@@ -312,7 +316,9 @@ init_per_testcase(version_option, Config0) ->
 				{user, User},
 				{password, Passwd},
 				{user_interaction, false},
-				{silently_accept_hosts, true}]),
+				{silently_accept_hosts, true},
+                                {save_accepted_host, false}
+                               ]),
     Sftp = {ChannelPid, Connection},
     [{sftp,Sftp}, {watchdog, Dog} | TmpConfig];
 
@@ -336,7 +342,8 @@ init_per_testcase(Case, Config00) ->
 					   [{user, User},
 					    {password, Passwd},
 					    {user_interaction, false},
-					    {silently_accept_hosts, true}
+					    {silently_accept_hosts, true},
+                                            {save_accepted_host, false}
 					    | PktSzOpt
 					   ]
 					  ),
@@ -349,7 +356,8 @@ init_per_testcase(Case, Config00) ->
 		{ok, ChannelPid, Connection} = 
 		    ssh_sftp:start_channel(Host, 
 					   [{user_interaction, false},
-					    {silently_accept_hosts, true}
+					    {silently_accept_hosts, true},
+                                            {save_accepted_host, false}
 					    | PktSzOpt
 					   ]),
 		Sftp = {ChannelPid, Connection},
@@ -556,26 +564,62 @@ links(Config) when is_list(Config) ->
 retrieve_attributes(Config) when is_list(Config) ->
     FileName = proplists:get_value(filename, Config),
     SftpFileName = w2l(Config, FileName),
-
     {Sftp, _} = proplists:get_value(sftp, Config),
-    {ok, FileInfo} = ssh_sftp:read_file_info(Sftp, SftpFileName),
-    {ok, NewFileInfo} = file:read_file_info(FileName),
-
-    %% TODO comparison. There are some differences now is that ok?
-    ct:log("SFTP: ~p   FILE: ~p~n", [FileInfo, NewFileInfo]).
+    {ok, SftpFileInfo} = ssh_sftp:read_file_info(Sftp, SftpFileName),
+    {ok, FileFileInfo} = file:read_file_info(FileName),
+    ct:log("ssh_sftp:read_file_info(): ~p~n"
+           "file:read_file_info(): ~p",
+           [SftpFileInfo, FileFileInfo]),
+    {ExpectedUid, ExpectedGid} =
+        case {os:type(), proplists:get_value(group,Config)} of
+            {{win32, _}, openssh_server} ->
+                %% Windows compiled Erlang is expected will return 0;
+                %% but when Erlang(Windows) client interacts with
+                %% OpenSSH server - value 1000 is received by client
+                %% over SFTP (because OpenSSH is compiled for Linux
+                %% and runs on WSL)
+                {1000, 1000};
+            _ ->
+                {FileFileInfo#file_info.uid, FileFileInfo#file_info.gid}
+        end,
+    ?assertEqual(ExpectedUid, SftpFileInfo#file_info.uid),
+    ?assertEqual(ExpectedGid, SftpFileInfo#file_info.gid),
+    ok.
 
 %%--------------------------------------------------------------------
 set_attributes(Config) when is_list(Config) ->
     FileName = proplists:get_value(testfile, Config),
     SftpFileName = w2l(Config, FileName),
-
     {Sftp, _} = proplists:get_value(sftp, Config),
     {ok,Fd} = file:open(FileName, write),
     io:put_chars(Fd,"foo"),
-    ok = ssh_sftp:write_file_info(Sftp, SftpFileName, #file_info{mode=8#400}),
-    {error, eacces} = file:write_file(FileName, "hello again"),
-    ok = ssh_sftp:write_file_info(Sftp, SftpFileName, #file_info{mode=8#600}),
-    ok = file:write_file(FileName, "hello again").
+    TestWriting =
+        fun(FInfo) ->
+                ok = ssh_sftp:write_file_info(Sftp, SftpFileName,
+                                              FInfo#file_info{mode=8#400}),
+                {error, eacces} = file:write_file(FileName, "hello again"),
+                ok = ssh_sftp:write_file_info(Sftp, SftpFileName,
+                                              FInfo#file_info{mode=8#600}),
+                ok = file:write_file(FileName, "hello again")
+        end,
+    TestWriting(#file_info{}),
+    IsErlangServer =
+        fun() ->
+                TcGroupPath = proplists:get_value(tc_group_path, Config),
+                {_, Path} = lists:unzip(lists:flatten(TcGroupPath)),
+                lists:member(erlang_server, Path)
+        end,
+    case IsErlangServer() of
+        true ->
+            ct:log("Testing with writing a complete #file_info record"),
+            {ok, FileInfo} = file:read_file_info(SftpFileName),
+            TestWriting(FileInfo);
+        _ ->
+            %% with OpenSSH daemon started by other user above instruction end
+            %% up with permission denied
+            ok
+    end,
+    ok.
 
 %%--------------------------------------------------------------------
 file_owner_access(Config) when is_list(Config) ->
@@ -736,7 +780,8 @@ start_channel_sock(Config) ->
 	end,
 
     Opts = [{user_interaction, false},
-	    {silently_accept_hosts, true}
+	    {silently_accept_hosts, true},
+            {save_accepted_host, false}
 	    | LoginOpts],
 
     {Host,Port} = proplists:get_value(peer, Config),
@@ -789,7 +834,9 @@ sftp_nonexistent_subsystem(Config) when is_list(Config) ->
 			       [{user_interaction, false},
 				{user, User},
 				{password, Passwd},
-				{silently_accept_hosts, true}]).
+				{silently_accept_hosts, true},
+                                {save_accepted_host, false}
+                               ]).
 
 %%--------------------------------------------------------------------
 version_option(Config) when is_list(Config) ->
